from matchbook.endpoints.baseendpoint import BaseEndpoint
from matchbook.exceptions import AuthError
from matchbook.utils import logger


class Logout(BaseEndpoint):

    def __call__(self, session=None):
        response = self.request("DELETE", self.client.urn_main, 'security/session', data=self.data, session=session)
        self.client.set_session_token(None, None)
        logger.info('Logout Successful. Session Token: {}'.format(self.client.session_token))

    @property
    def data(self):
        return {'username': self.client.username, 'password': self.client.password}




import datetime

from matchbook import resources
from matchbook.endpoints.baseendpoint import BaseEndpoint
from matchbook.enums import Boolean, Side, MarketNames, MarketType, MarketStates
from matchbook.utils import clean_locals


class MarketData(BaseEndpoint):

    def get_events(self, event_id=None, before=None, after=None, sport_ids=None, category_ids=None,
                   states=MarketStates.All, tag_url_names=None, per_page=500, offset=0,
                   include_event_participants=Boolean.T, price_depth=3, side=Side.All,
                   minimum_liquidity=None, session=None):
        """
        Get paginated events. Results can be filtered using various different parameters.

        :param event_id: specific event id. Default None.
        :type event_id: int
        :param after: event start time lower cutoff. Default None.
        :type after: UNIX timestamp
        :param before: event start time upper cutoff. Default None.
        :type before: UNIX timestamp
        :param category_ids: filter results by category id. Default None.
        :type category_ids: comma separated string
        :param sport_ids: filter results by sports id(s). Default None.
        :type sport_ids: comma separated string
        :param states: filter results by event state  or comma separated string of types. Default None.
        :type states: matchbook.enums.MarketStates
        :param tag_url_names:Only events with tags having url-name in the provided list are included in the response.
        :type tag_url_names: comma separated string
        :param per_page: number of results to show in a single result. Max=500. Default 20.
        :type per_page: int
        :param offset: starting page of results to show. Default 0.
        :type offset: int
        :param include_event_participants: A boolean indicating whether to return the event participants information
        :type include_event_participants: matchbook.enums.Boolean
        :param price_depth: max depth to be returned for prices. Default 3.
        :type price_depth: int
        :param side: filter results by side (dependent on exchange-type). Default None.
        :type side: matchbook.enums.Side
        :param minimum_liquidity: Only prices with available-amount greater than or equal to this value are included.
        :type minimum_liquidity: float
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Breakdown to each runner if they are included.
        :rtype: json
        :raises: matchbook.exceptions.ApiError
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        method = 'events'
        params['odds-type'] = self.client.odds_type
        params['exchange-type'] = self.client.exchange_type
        params['currency'] = self.client.currency
        if event_id:
            method = 'events/%s' % event_id
            del_keys = ['after', 'before', 'category-ids', 'sport-ids', 'states', 'per-page', 'offset', 'tag-url-names']
            params = {k: v for k, v in params.items() if k not in del_keys}
            response = self.request("GET", self.client.urn_edge, method, params=params, session=session)
            response = response.json().get('event', response.json())
        else:
            response = self.request(
                "GET", self.client.urn_edge, method, params=params, target='events', session=session
            )
        return self.process_response(response, resources.Event, date_time_sent, datetime.datetime.utcnow())

    def get_markets(self, event_id, market_id=None, names=MarketNames.All, types=MarketType.All, offset=0, per_page=500,
                    states=MarketStates.All, price_depth=3, side=Side.Default, minimum_liquidity=None, session=None):
        """
        Get paginated markets for an event specified by the event_id.

        :param event_id: specific event id.
        :type event_id: int
        :param market_id: specific market id to pull data for.
        :type market_id: int
        :param states: filter results by market state or a comma separated string of states. Default 'open', 'suspended'
        :type states: matchbook.enums.MarketStates
        :param types: filter results by market type or a comma separated string of types. Default None.
        :type types: matchbook.enums.MarketType
        :param names: filter results by market name. Default None.
        :type names: matchbook.enums.MarketNames
        :param per_page: number of results to show in a single result. Max=500. Default 20.
        :type per_page: int
        :param offset: starting page of results to show. Default 0.
        :type offset: int
        :param price_depth: max depth to be returned for prices. Default 3.
        :type price_depth: int
        :param side: filter results by side (dependent on exchange-type). Default None.
        :type side: matchbook.enums.Side
        :param minimum_liquidity: Only prices with available-amount greater than or equal to this value are included.
        :type minimum_liquidity: float
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Breakdown of each runner if they are included.
        :rtype: json
        :raises: matchbook.exceptions.ApiError
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        params['odds-type'] = self.client.odds_type
        params['exchange-type'] = self.client.exchange_type
        params['currency'] = self.client.currency
        method = 'events/%s/markets' % event_id
        if market_id:
            method = 'events/%s/markets/%s' % (event_id, market_id)
            del_keys = ['names', 'types', 'per-page', 'offset', 'states']
            params = {k: v for k, v in params.items() if k not in del_keys}
            response = self.request('GET', self.client.urn_edge, method, params=params, session=session)
            response = response.json().get('market', response.json())
        else:
            response = self.request(
                "GET", self.client.urn_edge, method, params=params, target='markets', session=session
            )
        return self.process_response(response, resources.Market, date_time_sent, datetime.datetime.utcnow())

    def get_runners(self, event_id, market_id, runner_id=None, states=MarketStates.All, include_withdrawn=Boolean.T,
                    include_prices=Boolean.T, price_depth=3, side=Side.All, minimum_liquidity=None, session=None):
        """
        Get runner data for an event and market specified by their ids.

        :param event_id: specific event id.
        :type event_id: int
        :param market_id: specific market id to pull data for.
        :type market_id: int
        :param runner_id: specific runner to pull data for.
        :type runner_id: int
        :param states: filter results by runner state or a comma separated string of states. Default 'open', 'suspended'
        :param include_withdrawn: boolean for returning or not the withdrawn runners in the response.
        :type include_withdrawn: matchbook.enums.Boolean
        :param include_prices: boolean indicating whether to return the prices for the runners.
        :type include_prices: matchbook.enums.Boolean
        :type states: matchbook.enums.MarketStates
        :param price_depth: max depth to be returned for prices. Default 3.
        :type price_depth: int
        :param side: filter results by side (dependent on exchange-type). Default None.
        :type side: matchbook.enums.Side
        :param minimum_liquidity: Only prices with available-amount greater than or equal to this value are included.
        :type minimum_liquidity: float
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Breakdown of each runner if they are included.
        :rtype: json
        :raises: matchbook.exceptions.ApiError
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        params['odds-type'] = self.client.odds_type
        params['exchange-type'] = self.client.exchange_type
        params['currency'] = self.client.currency
        method = 'events/%s/markets/%s/runners' % (event_id, market_id)
        if runner_id:
            method = 'events/%s/markets/%s/runners/%s' % (event_id, market_id, runner_id)
            del_keys = ['include-withdraw', 'states']
            params = {k: v for k, v in params.items() if k not in del_keys}
            response = self.request('GET', self.client.urn_edge, method, params=params, session=session)
            response = response.json().get('runner', response.json())
        else:
            response = self.request(
                'GET', self.client.urn_edge, method, params=params, target='runners', session=session
            )
        return self.process_response(response, resources.Runner, date_time_sent, datetime.datetime.utcnow())

    def get_popular_markets(self, price_depth=3, side=Side.All, minimum_liquidity=None,
                            old_format=Boolean.F, session=None):
        """
        Get popular markets as defined by matchbook.

        :param price_depth: max depth to be returned for prices. Default 3.
        :type price_depth: int
        :param side: filter results by side (dependent on exchange-type). Default None.
        :type side: matchbook.enums.Side
        :param minimum_liquidity: Only prices with available-amount greater than or equal to this value are included.
        :type minimum_liquidity: float
        :param old_format:
        :type old_format:
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Breakdown of each runner if they are included.
        :rtype: json
        :raises: matchbook.exceptions.ApiError
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        params['odds-type'] = self.client.odds_type
        params['exchange-type'] = self.client.exchange_type
        params['currency'] = self.client.currency
        response = self.request('GET', self.client.urn_edge, 'popular-markets', params=params, session=session)
        return self.process_response(
            response.json().get('markets', response.json()), resources.Market,
            date_time_sent, datetime.datetime.utcnow()
        )



import datetime

from matchbook import resources
from matchbook.utils import clean_locals
from matchbook.endpoints.baseendpoint import BaseEndpoint
from matchbook.enums import TransactionCategories, TransactionTypes, PeriodFilter


class Reporting(BaseEndpoint):

    def get_old_transactions_report(self, offset=0, per_page=500, after=None, before=None, period=PeriodFilter.Default,
                                    categories=TransactionCategories.Exchange, session=None):
        """
        Get paginated historical transactions, filtered by arguments specified

        :param offset: starting point from which the paginated results begin. Default 0.
        :type offset: int
        :param per_page: number of bets to be returned per call, max 500. Default 20.
        :type per_page: int
        :param after: event start time lower cutoff. Default 0.
        :type after: UNIX timestamp
        :param before: event start time upper cutoff. Default current time.
        :type before: UNIX timestamp
        :param period: filter for the amount of time to include in settlement search.
        :type period: matchbook.enums.PeriodFilter
        :param categories: where the transaction was incurred, casino or exchange.
        :type categories: matchbook.enums.TransactionCategories
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Historical transaction info.
        :rtype: json
        :raises: matchbook.exceptions.ApiError
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        response = self.request(
            'GET', self.client.urn_main, 'reports/transactions', params=params, target='transactions', session=session
        )
        return self.process_response(
            response, resources.TransactionReport,
            date_time_sent, datetime.datetime.now()
        )

    def get_new_transactions_report(self, transaction_type=TransactionTypes.All, offset=0, per_page=500, 
                                    after=None, before=None, session=None):
        # TODO: Map resource and get example data for data_structures
        """
        Get paginated historical transactions, filtered by arguments specified

        :param offset: starting point from which the paginated results begin. Default 0.
        :type offset: int
        :param per_page: number of bets to be returned per call, max 500. Default 20.
        :type per_page: int
        :param after: event start time lower cutoff. Default 0.
        :type after: UNIX timestamp
        :param before: event start time upper cutoff. Default current time.
        :type before: UNIX timestamp
        :param transaction_type: filter by type of transaction.
        :type transaction_type: matchbook.enums.TransactionTypes
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Historical transaction info.
        :rtype: json
        :raises: matchbook.exceptions.ApiError
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        response = self.request(
            'GET', self.client.urn_edge, 'reports/v1/transactions', params=params,
            target='transactions', session=session
        )
        return self.process_response(response, resources.TransactionReport, date_time_sent, datetime.datetime.now())

    def get_current_offers(self, sport_ids=None, event_ids=None, market_ids=None, offset=0, per_page=500, session=None):
        """
        Get a list of current offers i.e. offers on markets yet to be settled.
        
        :param sport_ids: operate only on orders on specified sports.
        :type sport_ids: comma separated string
        :param event_ids: operate only on orders on specified events.
        :type event_ids: comma separated string
        :param market_ids: operate only on orders on specified markets.
        :type market_ids: comma separated string
        :param offset: starting point of results. Default 0.
        :type offset: int
        :param per_page: no. of offers returned in a single response, Max 500. Default 20.
        :type per_page: int
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Orders data
        :raises: MatchbookAPI.bin.exceptions.ApiError

        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        response = self.request(
            'GET', self.client.urn_edge, 'reports/v1/offers/current', params=params, target='offers', session=session
        )
        return self.process_response(response, resources.Order, date_time_sent, datetime.datetime.now())

    def get_current_bets(self, sport_ids=None, event_ids=None, market_ids=None, offset=0, per_page=500, session=None):
        """
        Get a list of current bets i.e. offers that have been matched on markets yet to be settled.
        
        :param sport_ids: operate only on orders on specified sports.
        :type sport_ids: comma separated string
        :param event_ids: operate only on orders on specified events.
        :type event_ids: comma separated string
        :param market_ids: operate only on orders on specified markets.
        :type market_ids: comma separated string
        :param offset: starting point of results. Default 0.
        :type offset: int
        :param per_page: no. of offers returned in a single response, Max 500. Default 20.
        :type per_page: int
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Orders data
        :raises: MatchbookAPI.bin.exceptions.ApiError

        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        response = self.request(
            'GET', self.client.urn_edge, 'reports/v1/bets/current', params=params, target='bets', session=session
        )
        return self.process_response(response, resources.BetReport, date_time_sent, datetime.datetime.now())

    def get_settled_bets(self, sport_ids=None, event_ids=None, market_ids=None, before=None, after=None,
                         offset=0, per_page=500, session=None):
        """
        Get a list of settled bets.
        
        :param sport_ids: operate only on bets on specified sports.
        :type sport_ids: comma separated string
        :param event_ids: operate only on bets on specified events.
        :type event_ids: comma separated string
        :param market_ids: operate only on bets on specified markets.
        :type market_ids: comma separated string
        :param after: event start time lower cutoff. Default None.
        :type after: UNIX timestamp
        :param before: event start time upper cutoff. Default None.
        :type before: UNIX timestamp
        :param offset: starting point of results. Default 0.
        :type offset: int
        :param per_page: no. of offers returned in a single response, Max 500. Default 20.
        :type per_page: int
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Orders data
        :raises: MatchbookAPI.bin.exceptions.ApiError
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        response = self.request(
            'GET', self.client.urn_edge, 'reports/v1/bets/settled', params=params, target='bets', session=session
        )
        return self.process_response(response, resources.BetReport, date_time_sent, datetime.datetime.now())





class Side:
    '''
    Specify what order types to return in order requests, dependant on exchange type, Default will return all.

    :var back: orders backing the runner, suitable for exchange type back-lay.
    :var lay: orders backing the runner, suitable for exchange type back-lay.
    :var win: orders to win, suitable for exchange type binary.
    :var loss: orders to loss, suitable for exchange type binary.
    '''
    Back = 'back'
    Lay = 'lay'
    Win = 'win'
    Loss = 'loss'
    All = None
    Default = All


class ExchangeType:
    '''
    Format of odds/liquidity returned on both sides.
    '''
    Binary = 'binary'
    BackLay = 'back-lay'
    Default = BackLay


class Status:
    '''
    Specify the order status to return, Default will return all.

    :var matched:  Offer has been partially or fully matched. Returned status will be "matched" for fully matched or "open" for partially matched.
    :var unmatched: Offer still has some stake available to be matched but may be "open" or "paused".
    :var cancelled: Offer has been cancelled and can no longer be matched.
    :var open: Initial state of offer before being partially matched, fully matched or edited.
    :var paused: Offer still has some remaining stake but cannot be matched until unpaused.
    :var expired: Offer has expired and can no longer be matched.

    '''
    Matched = 'matched'
    Unmatched = 'unmatched'
    Cancelled = 'cancelled'
    Expired = 'expired'
    Open = 'open'
    Paused = 'paused'
    All = None
    Default = All


class OddsType:
    '''
    Odds format to be used/returned.

    :var US: american odds format.
    :var DECIMAL: decimal odds format.
    :var %: probability.
    :var HK: hong kong odds format.
    :var MALAY: malay odds format.
    :var INDO: indonesian odds format.

    '''
    US = 'US'
    Decimal = 'DECIMAL'
    Prob = '%'
    HongKong = 'HK'
    Malay = 'MALAY'
    Indo = 'INDO'
    Default = Decimal


class Currency:
    '''
    Currency selection for use in placing bets.

    :var EUR: EUR
    :var USD: USD
    :var GBP: GBP
    '''
    EUR = 'EUR'
    USD = 'USD'
    GBP = 'GBP'
    Default = EUR


class MarketStates:
    '''
    Filter by market state.

    :var open: market is currently open.
    :var suspended: market is currently suspended.
    '''
    Open = 'open'
    Suspended = 'suspended'
    All = None
    Default = All


class RunnerStates:
    '''
    Filter by runner state

    :var open: market is currently open.
    :var suspended: market is currently suspended.
    '''
    Open = 'open'
    Suspended = 'suspended'
    All = None
    Default = All


class MarketType:
    '''
    Filter by market type

    :var multirunner: market has multirunner type.
    :var binary: market has a binary outcome.
    '''
    MultipleRunners = 'multirunner'
    Binary = 'binary'
    All = None
    Default = All


class GradingType:
    '''
    Filter by grading type i.e. how a market settles.

    :var asian-handicap: A dual handicap market, where each handicap is graded as per point spread below.
    :var high-score-wins: Markets where the team scoring the most points or goals are graded as the winner.
    :var low-score-wins: Markets where the team scoring the least points or goals are graded as the winner.
    :var point-spread: Markets where the score in the game, and the handicap value, are factored in for grading purposes.
    :var point-total: Markets where the total of all scores in the game decide the grading.
    :var single-winner-wins: Other markets where the grading is decided manually.
    '''
    AsianHandicap = 'asian-handicap'
    HighScore = 'high-score-wins'
    LowScore = 'low-score-wins'
    PointSpread = 'point-spread'
    PointTotal = 'point-total'
    SingleWinner = 'single-winner-wins'
    All = None
    Default = All


class MarketOrder:
    '''
    Determine how results are ordered when returned.

    :var start asc: order by start time ascending.
    :var start desc: order by start time descending.
    '''
    Ascending = 'start asc'
    Descending = 'start desc'
    Default = Ascending


class Boolean:
    '''
    booleans in api are lower case string type rather than python boolean.
    '''
    T = 'true'
    F = 'false'


class PriceOrder:
    '''
    Determine how prices are ordered when returned.

    :var price asc: order by price ascending.
    :var price desc: oder by price descending.
    '''
    Ascending = 'price asc'
    Descending = 'price desc'
    Default = Descending


class MarketNames:
    '''
    Market names which can be used to filter results.

    :TODO Check all market names for validity.
    '''
    Match = 'match'
    PointSpread = 'point spread'
    OverUnder = 'over/under'
    GameLine = 'gameline'
    Match1H = 'match 1H'
    GameLine1H = 'gameline 1H'
    OverUnder1H = 'over/under 1H'
    Winner = 'Winner'
    ML = 'Money Line'
    ml = 'moneyline'
    gl = 'goal line'
    NumberRounds = 'Number of Rounds'
    TotalRounds = 'Total Rounds'
    Totals = 'totals'
    ThreeWay = 'Three Way'
    Total = 'Total'
    RaceWinner = 'Race Winner'
    All = None
    Default = All


class SequenceOrder:
    '''
    Determine how markets are ordered when returned.

    :var start asc: sort by start time ascending.
    :var start desc: sort by start time descending.
    :var seq asc: sort in sequential order ascending.
    :var seq desc: sort in sequential order descending.
    '''
    StartAscending = 'start asc'
    StartDescending = 'start desc'
    SeqAscending = 'seq asc'
    SeqDescending = 'seq desc'
    Default = SeqAscending


class SportsOrder:
    '''
    Determine how sports are ordered when returned.

    :var name asc: order alphabetically ascending.
    :var name desc: order alphabetically descending.
    :var id asc: order by id number ascending.
    :var id desc: order by id number descending.
    '''
    NameAsc = 'name asc'
    NameDesc = 'name desc'
    IDAsc = 'id asc'
    IDDEsc = 'id desc'
    Default = NameAsc


class SportStatus:
    '''
    Filter sport search by status of sport.

    :var active: there are markets active on this sport.
    :var pending: there are no active markets on this sport.
    '''
    Active = 'active'
    Pending = 'pending'
    Default = Active


class BetsOrder:
    '''
    Determine the order in which bet report is sorted.

    :var event asc: event name alphabetically ascending.
    :var event desc: event name alphabetically descending.
    :var selection asc: selection name alphabetically ascending.
    :var selection desc: selection name alphabetically descending.
    :var market asc: market name alphabetically ascending.
    :var market desc: market name alphabetically descending.
    :var bet-id asc: bet id numerically ascending.
    :var bet-id desc: bet id numerically descending.
    '''
    EventAsc = 'event asc'
    EventDesc = 'event desc'
    SelectionAsc = 'selection asc'
    SelectionDesc = 'selection desc'
    MarketAsc = 'market asc'
    MarketDesc = 'market desc'
    IDAsc = 'bet-id asc'
    IDDesc = 'bet-id desc'


class BetStatusFilter:
    '''
    Filter bet report to return matched bets only or now.

    :var true: return only matched bets.
    :var false: return all bets.
    '''
    MatchedOnly = 'true'
    All = 'false'
    Default = MatchedOnly


class BetGrouping:
    '''
    Group betting report to return average odds and total stake on runners or all bets individually.

    :var true: group bets by runner and average odds, sum stakes
    :var false: return all bets individually
    '''
    GroupBets = 'true'
    IndividualBets = 'false'
    Default = IndividualBets


class SizeFilter:
    """
    Filter to only include data where a specified dataset meets this criteria.

    :var greater-than: values are greater than a specific cutoff.
    :var less-than: values are less than a specific cutoff.
    :var equals: values are equal to a specific value.
    """
    GreaterThan = 'greater-than'
    LessThan = 'less-than'
    Equal = 'equals'
    All = None
    Default = All


class PeriodFilter:
    """
    Period to filter reports to include.

    :var today: include only today.
    :var yesterday: include only yesterday
    :var 1-day: include one calendar day.
    :var 2-day: include two calendar day.
    :var week: include one calendar week.
    :var month: include one calendar month.
    :var 1-month: include one calendar month.
    :var 3-month: include three calendar months.
    """
    Today = 'today'
    Yesterday = 'yesterday'
    Days1 = '1-day'
    Days2 = '2-day'
    Week = 'week'
    Month = 'month'
    Month1 = '1-month'
    Month3 = '3-month'
    All = None
    Default = All


class TransactionCategories:
    """
    Categories a transaction can fall under.

    :var casino: transaction was made on the casino site.
    :var exchange: transaction was made on the exchange site.
    :var collosus: NFI.
    """
    Casino = 'casino'
    Exchange = 'exchange'
    Collosus = 'collosus'
    Default = Exchange


class TransactionTypes:
    """
    Types of transactions.
    
    :var
    """
    Payout = 'payout'
    Commission = 'commission'
    Transfer = 'transfer'
    Cancel = 'cancel'
    Manual = 'manual'
    Bonus = 'bonus'
    All = None
    Default = All


class AggregationType:
    """
    Method of aggregation to be used in grouping bets.
    """
    Average = 'average'
    Summary = 'summary'
    Default = Average



from matchbook.resources.baseresource import BaseResource


class BetReport(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'id': 'id',
            'event-id': 'event-id',
            'event-name': 'event-name',
            'exchange-type': 'exchange-type',
            'market-id': 'market-id',
            'market-type': 'market-type',
            'odds': 'odds',
            'offer-id': 'offer-id',
            'profit-and-loss': 'profit-and-loss',
            'runner-id': 'runner-id',
            'selection': 'selection',
            'side': 'side',
            'sport-id': 'sport-id',
            'stake': 'stake',
            'submitted-at': 'submitted-at',
        }
        datetime_attributes = (
            'submitted-at'
        )


class SettlementBet(BaseResource):
    class Meta(BaseResource.Meta):
        identifier = 'bets'
        attributes = {
            'profit-and-loss': 'profit-and-loss',
            'matched-time': 'matched-time',
            'odds': 'odds',
            'commission-rate': 'commission-rate',
            'in-play': 'in-play',
            'offer-id': 'offer-id',
            'id': 'id',
            'settled-time': 'settled-time',
            'stake': 'stake',
            'commission': 'commission'
        }


class SettlementSelection(BaseResource):
    class Meta(BaseResource.Meta):
        identifier = 'selections'
        attributes = {
            'name': 'name',
            'odds': 'odds',
            'profit-and-loss': 'profit-and-loss',
            'id': 'id',
            'side': 'side',
            'stake': 'stake',
            'commission': 'commission'
        }
        sub_resources = {
            'bets': SettlementBet
        }


class SettlementMarket(BaseResource):
    class Meta(BaseResource.Meta):
        identifier = 'markets'
        attributes = {
            'profit-and-loss': 'profit-and-loss',
            'net-win-commission': 'net-win-commission',
            'name': 'name',
            'id': 'id',
            'commission': 'commission',
            'stake': 'stake',
        }
        sub_resources = {
            'selections': SettlementSelection
        }


class SettlementEvent(BaseResource):
    class Meta(BaseResource.Meta):
        identifier = 'events'
        attributes = {
            'sport-url': 'sport-url',
            'start-time': 'start-time',
            'sport-name': 'sport-name',
            'name': 'name',
            'id': 'id',
            'sport-id': 'sport-id',
            'finished-dead-heat': 'finished-dead-heat'
        }
        sub_resources = {
            'markets': SettlementMarket
        }


class SettlementReport(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'offset': 'offset',
            'profit-and-loss': 'profit-and-loss',
            'odds-type': 'odds-type',
            'total': 'total',
            'per-page': 'per-page',
            'language': 'language',
            'overall-staked-amount': 'overall-staked-amount'
        }
        sub_resources = {
            'events': SettlementEvent
        }
        datetime_attributes = (
            'settled-at'
        )


class MarketSettlementReport(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'exchange-type': 'exchange-type',
            'odds': 'odds',
            'profit-and-loss': 'profit-and-loss',
            'runner-id': 'runner-id',
            'runner-name': 'runner-name',
            'side': 'side',
            'stake': 'stake',
            'settled-at': 'settled-at',
        }
        datetime_attributes = (
            'settled-at'
        )


class RunnerSettlementReport(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'bet-id': 'bet-id',
            'exchange-type': 'exchange-type',
            'market-id': 'market-id',
            'odds': 'odds',
            'offer-id': 'offer-id',
            'offer-type': 'offer-type',
            'placed-at': 'placed-at',
            'profit-and-loss': 'profit-and-loss',
            'runner-id': 'runner-id',
            'runner-name': 'runner-name',
            'settled-at': 'settled-at',
            'side': 'side',
            'stake': 'stake',
            'trade-type': 'trade-type',
        }
        datetime_attributes = (
            'settled-at',
            'placed-at'
        )


class CommissionReport(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'commission': 'commission',
            'commissionable-handle': 'commissionable-handle',
            'offer-type': 'offer-type',
            'placed-at': 'placed-at',
            'rate': 'rate',
            'runner-id': 'runner-id',
            'runner-name': 'runner-name',
            'settled-at': 'settled-at',
            'trade-type': 'trade-type',
        }
        datetime_attributes = (
            'settled-at',
            'placed-at'
        )


class TransactionReport(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'id': 'id',
            'balance': 'balance',
            'category': 'category',
            'debit': 'debit',
            'detail': 'detail',
            'time-settled': 'time-settled',
            'type': 'type',
        }
        datetime_attributes = (
            'time-settled'
        )


from matchbook.resources import BaseResource


class Price(BaseResource):
    class Meta(BaseResource.Meta):
        identifier = 'prices'
        attributes = {
            'available-amount': 'available-amount',
            'currency': 'currency',
            'decimal-odds': 'decimal-odds',
            'exchange-type': 'exchange-type',
            'odds': 'odds',
            'odds-type': 'odds-type',
            'side': 'side',
        }


class Runner(BaseResource):
    class Meta(BaseResource.Meta):
        identifier = 'runners'
        attributes = {
            'event-id': 'event-id',
            'event-participant-id': 'event-participant-id',
            'id': 'id',
            'market-id': 'market-id',
            'name': 'name',
            'status': 'status',
            'volume': 'volume',
            'withdrawn': 'withdrawn',
        }
        sub_resources = {
            'prices': Price,
        }


class Market(BaseResource):
    class Meta(BaseResource.Meta):
        identifier = 'markets'
        attributes = {
            'allow-live-betting': 'allow-live-betting',
            'back-overround': 'back-overround',
            'event-id': 'event-id',
            'grading-type': 'grading-type',
            'handicap': 'handicap',
            'id': 'id',
            'in-running-flag': 'in-running-flag',
            'lay-overround': 'lay-overround',
            'market-ids': 'market-ids',
            'name': 'name',
            'runner-ids': 'runner-ids',
            'start': 'start',
            'status': 'status',
            'type': 'type',
            'market-type': 'market-type',
            'volume': 'volume',
        }
        sub_resources = {
            'runners': Runner,
        }
        datetime_attributes = (
            'start'
        )


class EventMeta(BaseResource):
    class Meta(BaseResource.Meta):
        identifier = 'meta-tags'
        attributes = {
            'id': 'id',
            'name': 'name',
            'type': 'type',
            'url-name': 'url-name',
        }


class Event(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'allow-live-betting': 'allow-live-betting',
            'category-id': 'category-id',
            'id': 'id',
            'in-running-flag': 'in-running-flag',
            'name': 'name',
            'sport-id': 'sport-id',
            'start': 'start',
            'status': 'status',
            'volume': 'volume',
        }
        sub_resources = {
            'markets': Market,
            'meta-tags': EventMeta,
        }
        datetime_attributes = (
            'start'
        )



import requests
import os

from matchbook.exceptions import PasswordError
from matchbook.enums import ExchangeType, OddsType, Currency


class BaseClient(object):

    def __init__(self, username, password=None, locale=None):
        """
        :param username: Matchbook username.
        :param password: Password for supplied username, if None will look in for MATCHBOOK_PW in env variables.
        """
        self.username = username
        self.password = password
        self.locale = locale
        self.url = 'https://matchbook.com'
        self.url_beta = 'https://beta.matchbook.com'
        self.urn_main = '/bpapi/rest/'
        self.urn_edge = '/edge/rest/'
        self.session = requests.Session()
        self.session_token = None
        self.user_id = None
        self.exchange_type = ExchangeType.BackLay
        self.odds_type = OddsType.Decimal
        self.currency = Currency.EUR
        self.get_password()

    def set_session_token(self, session_token, user_id):
        """Sets session token.
        
        :param session_token: Session token from request.
        :param user_id: User Id from the request.
        """
        self.session_token = session_token
        self.user_id = user_id

    def get_password(self):
        """If password is not provided will look in environment
        variables for username+'password'
        """
        if self.password is None:
            if os.environ.get('MATCHBOOK_PW'):
                self.password = os.environ.get('MATCHBOOK_PW')
            else:
                raise PasswordError()

    @property
    def headers(self):
        """Set headers to be used in API requests."""
        return {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
        }



import datetime

from matchbook.utils import clean_locals
from matchbook.endpoints.baseendpoint import BaseEndpoint
from matchbook import resources
from matchbook.enums import SportsOrder, MarketStates


class ReferenceData(BaseEndpoint):

    def get_currencies(self, session=None):
        """
        Get the accepted currencies on betting platform.
        
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Supported currencies.
        :rtype: json
        :raises: MatchbookAPI.bin.exceptions.ApiError
        """
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", self.client.urn_main, 'lookups/currencies', session=session)
        return self.process_response(
            response.json().get('currencies', []), resources.Currencies, date_time_sent, datetime.datetime.utcnow()
        )

    def get_sports(self, status=MarketStates.All, order=SportsOrder.NameAsc, per_page=500, session=None):
        """
        Lookup all sports, filter for active only/all.

        :param status: filter results by sport status, default 'active'.
        :type status: MatchbookAPI.bin.enums.SportStatus
        :param order: order in which results are returned, default 'name asc'.
        :type order: MatchbookAPI.bin.enums.SportsOrder
        :param per_page: no. of sports returned in a single response, Max 500. Default 20.
        :type per_page: int
        :param session: requests session to be used.
        :type session: requests.Session
        :return: Sports that are active on the betting platform.
        :rtype: json
        :raises: MatchbookAPI.bin.exceptions.ApiError
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", self.client.urn_edge, 'lookups/sports', params=params, session=session)
        return self.process_response(
            response.json().get('sports', []), resources.SportsDetails, date_time_sent, datetime.datetime.utcnow()
        )

    def get_oddstype(self, session=None):
        """
        Get the accepted odds-types.

        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Supported odds-types.
        :rtype: json
        :raises: MatchbookAPI.bin.exceptions.ApiError
        """
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", self.client.urn_main, 'lookups/odds-types', session=session)
        return self.process_response(
            response.json().get('odds-types', []), resources.OddsType, date_time_sent, datetime.datetime.utcnow()
        )

    def get_countries(self, session=None):
        """
        Get the countries and their relevant codes and ids.

        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Countries and their details.
        :rtype: json
        :raises: MatchbookAPI.bin.exceptions.ApiError
        """
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", self.client.urn_main, 'lookups/countries', session=session)
        return self.process_response(
            response.json().get('countries', []), resources.Countries, date_time_sent, datetime.datetime.utcnow()
        )

    def get_regions(self, country_id, session=None):
        """
        Get regions for a given country and their relevant codes and ids.

        :param country_id: id of the country whose regions we want to get.
        :type country_id: int
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Countries and their details.
        :rtype: json
        :raises: MatchbookAPI.bin.exceptions.ApiError
        """
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", self.client.urn_main, 'lookups/regions/%s' % country_id, session=session)
        return self.process_response(
            response.json().get('countries', []), resources.Regions, date_time_sent, datetime.datetime.utcnow()
        )

    def get_navigation(self, offset=0, per_page=500, session=None):
        # TODO: Map meta-tags to a resource.
        """
        Get page navigation tree breakdown.

        :param offset: starting point of results. Default 0.
        :type offset: int
        :param per_page: no. of offers returned in a single response, Max 500. Default 20.
        :type per_page: int
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Orders data
        :raises: MatchbookAPI.bin.exceptions.ApiError
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", self.client.urn_edge, 'navigation', params=params, session=session)
        return self.process_response(
            response.json().get('countries', []), resources.MetaTags, date_time_sent, datetime.datetime.utcnow()
        )



import datetime

from matchbook.endpoints.baseendpoint import BaseEndpoint
from matchbook import resources
from matchbook.enums import Side, Status, AggregationType
from matchbook.utils import clean_locals


class Betting(BaseEndpoint):

    def get_orders(self, event_ids=None, market_ids=None, runner_ids=None, offer_id=None, offset=0, per_page=500,
                   interval=None, side=Side.Default, status=Status.Default, session=None):
        """
        Get all orders which fit the argument filters.

        :param event_ids: operate only on orders on specified events.
        :type event_ids: comma separated string
        :param market_ids: operate only on orders on specified markets.
        :type market_ids: comma separated string
        :param runner_ids: operate only on orders on specified runners.
        :type runner_ids: comma separated string
        :param offer_id: specific order id to use.
        :param offset: starting point of results. Default 0.
        :type offset: int
        :param per_page: no. of offers returned in a single response, Max 500. Default 20.
        :type per_page: int
        :param interval: check for orders updated/created in last x seconds, status param must be 'matched'.
        :type interval: int
        :param side: filter results by side (dependent on exchange-type). Default None.
        :type side: MatchbookAPI.bin.enums.Side
        :param status: operate only on orders with specified status. Default None.
        :type status: MatchbookAPI.bin.enums.Status
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Orders data
        :raises: MatchbookAPI.bin.exceptions.ApiError

        """
        params = clean_locals(locals())
        params['exchange-type'] = self.client.exchange_type
        method = 'offers'
        date_time_sent = datetime.datetime.utcnow()
        if offer_id:
            method = 'offers/{0}'.format(offer_id)
            params = {'odds-type': self.client.odds_type}
            response = self.request("GET", self.client.urn_edge, method, params=params, session=session).json()
        else:
            response = self.request(
                "GET", self.client.urn_edge, method, params=params, target='offers', session=session
            )
        date_time_received = datetime.datetime.utcnow()
        return self.process_response(response, resources.Order, date_time_sent, date_time_received)

    def send_orders(self, runner_id, odds, side, stake, temp_id=None, session=None):
        """
        Place an order(s) on a runner, multiple orders can be places by providing lists of the required arguments.

        :param runner_id: runner(s) on which to place bets.
        :type runner_id: int
        :param odds: odds at which we wish to place the bet.
        :type odds: float
        :param side: The type of bet to place, dependent on exchange.
        :type side: MatchbookAPI.bin.enums.Side
        :param stake: amount in account currency to place on the bet.
        :type stake: float
        :param temp_id: A helper ID generated by the client to help understand the correlation between multiple submitted offers and their responses.
        :type temp_id: str
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Orders responses, i.e. filled or at exchange or errors.
        :raises: MatchbookAPI.bin.exceptions.ApiError

        """
        date_time_sent = datetime.datetime.utcnow()
        params = {
            'offers': [],
            'odds-type': self.client.odds_type,
            'exchange-type': self.client.exchange_type,
            'currency': self.client.currency,
        }
        if isinstance(runner_id, list):
            if isinstance(temp_id, list):
                for i, _ in enumerate(runner_id):
                    params['offers'].append({'runner-id': runner_id[i], 'side': side[i], 'stake': stake[i],
                                             'odds': odds[i], 'temp-id': temp_id[i]})
            else:
                for i, _ in enumerate(runner_id):
                    params['offers'].append({'runner-id': runner_id[i], 'side': side[i], 'stake': stake[i],
                                             'odds': odds[i]})
        else:
            params['offers'].append({'runner-id': runner_id, 'side': side, 'stake': stake, 'odds': odds})
        method = 'offers'
        response = self.request("POST", self.client.urn_edge, method, data=params, session=session)
        date_time_received = datetime.datetime.utcnow()
        return self.process_response(
            response.json().get('offers', []), resources.Order, date_time_sent, date_time_received
        )

    def get_agg_matched_bets(self, event_ids=None, market_ids=None, runner_ids=None, side=None, offset=0, per_page=500,
                             aggregation_type=AggregationType.Default, session=None):
        # TODO: Make aggregate matched bets resource
        """
         Get matched bets aggregated.

        :param event_ids: operate only on orders on specified events.
        :type event_ids: comma separated string
        :param market_ids: operate only on orders on specified markets.
        :type market_ids: comma separated string
        :param runner_ids: operate only on orders on specified runners.
        :type runner_ids: comma separated string
        :param offset: starting point of results. Default 0.
        :type offset: int
        :param per_page: no. of offers returned in a single response, Max 500. Default 20.
        :type per_page: int
        :param side: filter results by side (dependent on exchange-type). Default None.
        :type side: MatchbookAPI.bin.enums.Side
        :param aggregation_type: how to aggregate bets
        :type aggregation_type: matchbook.enums.AggregationType
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Orders data
        :raises: MatchbookAPI.bin.exceptions.ApiError

         """
        date_time_sent = datetime.datetime.utcnow()
        params = clean_locals(locals())
        method = 'bets/matched/aggregated'
        response = self.request("GET", self.client.urn_edge, method, params=params, target='bets', session=session)
        date_time_received = datetime.datetime.utcnow()
        return self.process_response(
            response, resources.MatchedBets, date_time_sent, date_time_received
        )

    def get_positions(self, event_ids=None, market_ids=None, runner_ids=None, offset=0, per_page=500, session=None):
        #TODO: Make positions resource
        """
        Get potential profit or loss on each runner.
        
        :param event_ids: operate only on orders on specified events.
        :type event_ids: comma separated string
        :param market_ids: operate only on orders on specified markets.
        :type market_ids: comma separated string
        :param runner_ids: operate only on orders on specified runners.
        :type runner_ids: comma separated string
        :param offset: starting point of results. Default 0.
        :type offset: int
        :param per_page: no. of offers returned in a single response, Max 500. Default 20.
        :type per_page: int
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Orders data
        :raises: MatchbookAPI.bin.exceptions.ApiError
        """
        date_time_sent = datetime.datetime.utcnow()
        params = clean_locals(locals())
        method = 'accounts/positions'
        response = self.request("GET", self.client.urn_edge, method, params=params, session=session)
        date_time_received = datetime.datetime.utcnow()
        return self.process_response(
            response.json().get('bets', []), resources.Order, date_time_sent, date_time_received
        )

    def amend_orders(self, order_id, odds, side, stake, session=None):
        """
        Adjust/Update an order(s), multiple orders can be adjusted by providing lists of the required arguments.

        :param order_id: order id to adjust.
        :type order_id: int
        :param odds: odds at which we wish to place the bet.
        :type odds: float
        :param side: back,lay|win,lose side to place bets on.
        :type side: MatchbookAPI.bin.enums.Side
        :param stake: amount in account currency to place on the bet.
        :type stake: float
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Orders responses, i.e. filled or at exchange or errors.
        :raises: MatchbookAPI.bin.exceptions.ApiError

        """
        date_time_sent = datetime.datetime.utcnow()
        params = {
            'offers': [],
            'odds-type': self.client.odds_type,
            'exchange-type': self.client.exchange_type,
            'currency': self.client.currency,
        }
        if isinstance(order_id, list):
            method = 'offers'
            for i, _ in enumerate(order_id):
                params['offers'].append({'id': order_id[i], 'side': side[i], 'stake': stake[i], 'odds': odds[i]})
        else:
            method = 'offers/{}'.format(order_id)
            del params['offers']
            params['stake'] = stake
            params['odds'] = odds
        response = self.request('PUT', self.client.urn_edge, method, data=params, session=session)
        date_time_received = datetime.datetime.utcnow()
        return self.process_response(
            response.json().get('offers', response.json()), resources.Order, date_time_sent, date_time_received
        )

    def delete_bulk_orders(self, event_ids=None, market_ids=None, runner_ids=None, offer_ids=None, session=None):
        """
        Delete all orders which fit the argument filters.

        :param event_ids: bulk delete orders on specified events.
        :type event_ids: comma separated string
        :param market_ids: bulk delete orders on specified markets.
        :type market_ids: comma separated string
        :param runner_ids: bulk delete orders on specified runners.
        :type runner_ids: comma separated string
        :param offer_ids: delete specific order id. Max offerids in one delete request is 25
        :type offer_ids: comma separated string
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: orders deletion report.
        :raises: MatchbookAPI.bin.exceptions.ApiError
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        method = 'offers'
        response = self.request('DELETE', self.client.urn_edge, method, data=params, session=session)
        date_time_received = datetime.datetime.utcnow()
        return self.process_response(
            response.json().get('offers', []), resources.Order, date_time_sent, date_time_received
        )

    def delete_order(self, offer_id, session=None):
        """
        Delete all orders which fit the argument filters.

        :param offer_id: delete specific order id.
        :type offer_id: int
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: order deletion report.
        :raises: MatchbookAPI.bin.exceptions.ApiError

        """
        date_time_sent = datetime.datetime.utcnow()
        method = 'offers/{}'.format(offer_id)
        response = self.request('DELETE', self.client.urn_edge, method, session=session)
        date_time_received = datetime.datetime.utcnow()
        return self.process_response(
            response.json(), resources.Order, date_time_sent, date_time_received
        )



from matchbook.resources.baseresource import BaseResource


class SportsDetails(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'id': 'id',
            'name': 'name',
            'type': 'type',
        }


class OddsType(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'odds-type': 'odds-type',
            'odds-type-id': 'odds-type-id',
        }


class Currencies(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'currency-id': 'currency-id',
            'long-name': 'long-name',
            'short-name': 'short-name',
        }


class Countries(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'country-id': 'country-id',
            'name': 'name',
            'country-code': 'country-code',
        }


class Regions(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'region-id': 'region-id',
            'name': 'name',
        }


class MetaTags(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {

        }



from matchbook.resources.baseresource import BaseResource

from matchbook.resources.accountresources import (
    AccountDetails,
    AccountBalance,
    AccountTransfer,
)

from matchbook.resources.bettingresources import (
    Order,
    MatchedBets,
    Position,
)

from matchbook.resources.reportresources import (
    BetReport,
    SettlementReport,
    MarketSettlementReport,
    RunnerSettlementReport,
    CommissionReport,
    TransactionReport,
    SettlementEvent
)

from matchbook.resources.referencedataresources import (
    SportsDetails,
    OddsType,
    Currencies,
    Countries,
    Regions,
    MetaTags,
)

from matchbook.resources.marketdataresources import (
    Event,
    Market,
    EventMeta,
    Runner,
    Price,
)


from matchbook.endpoints.login import Login
from matchbook.endpoints.keepalive import KeepAlive
from matchbook.endpoints.logout import Logout
from matchbook.endpoints.betting import Betting
from matchbook.endpoints.account import Account
from matchbook.endpoints.marketdata import MarketData
from matchbook.endpoints.referencedata import ReferenceData
from matchbook.endpoints.reporting import Reporting




import json

if not hasattr(json, 'JSONDecodeError'):
    json.JSONDecodeError = ValueError
else:
    from json.decoder import JSONDecodeError


class MBError(Exception):
    pass


class NotLoggedIn(MBError):
    pass


class AuthError(MBError):
    def __init__(self, response):
        self.response = response
        self.status_code = response.status_code
        try:
            self.message = response.json().get('errors')[0].get('messages')
        except:
            self.message = 'UNKNOWN'
        super(AuthError, self).__init__(self.message)


class ApiError(MBError):
    def __init__(self, response):
        self.response = response
        self.status_code = response.status_code
        try:
            error_data = response.json().get('errors')
            self.message = error_data[0].get('messages', 'UNKNOWN')

        except (KeyError, JSONDecodeError, TypeError):
            self.message = 'UNKNOWN'
            print(response)
        super(ApiError, self).__init__(self.message)


class PasswordError(MBError):
    """Exception raised if password is not found"""

    def __init__(self):
        message = 'Password not found in environment variables, add or pass to APIClient'
        super(PasswordError, self).__init__(message)


class StatusCodeError(MBError):

    def __init__(self, status_code):
        message = 'Status code error: %s' % status_code
        super(StatusCodeError, self).__init__(message)





import sys
import datetime
import logging
from matchbook.exceptions import ApiError

LOG_FORMAT = '%(asctime)s - %(levelname)s - %(module)s - %(funcName)s - %(message)s'
logging.basicConfig(
    level=logging.DEBUG,
    handlers=[
        logging.StreamHandler(sys.stdout)
    ],
    format=LOG_FORMAT,
)

logger = logging.getLogger(__name__)


def clean_time(col):
    """
    Parse a UTC time column to datetime.

    :param col: column to be parsed.
    :returns: datetime column
    :rtype: series

    """
    return col.apply(lambda x: datetime.datetime.strptime(x, '%Y-%m-%dT%H:%M:%S.%fZ'))


def filter_dicts(d):
    """
    Filter dict to remove None values.

    :param d: data to filter
    :type d: dict
    :returns: filtered data
    :rtype: dict

    """
    return dict((k, v) for k, v in d.items() if v is not None)


def clean_locals(params):
    """
    Clean up locals dict, remove empty and self params.

    :params params: locals dicts from a function.
    :type params: dict
    :returns: cleaned locals dict to use as params for functions
    :rtype: dict
    """
    clean_params = dict((k, v) for k, v in params.items() if v is not None and k != 'self' and k != 'session')
    for k, v in clean_params.items():
        if '_' in k:
            new_key = k.replace('_', '-')
            clean_params[new_key] = v
            clean_params.pop(k)
    return clean_params


def check_call_complete(response):
    return response.get('total', 0) < response.get('per-page', 20)


def check_status_code(response, codes=None):
    """Checks response.status_code is in codes
    :param response: Requests response
    :param codes: List of accepted codes or callable
    :raises: StatusCodeError if code invalid
    """
    codes = codes or [200]
    if response.status_code not in codes:
        raise ApiError(response)



import datetime

from matchbook.endpoints.baseendpoint import BaseEndpoint
from matchbook import resources
from matchbook.utils import clean_locals


class Account(BaseEndpoint):

    def get_account(self, balance_only=True, session=None):
        """
        Get account information for logged in user.

        :param balance_only: retrieve only account balance info subset or not.
        :type balance_only: bool
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Returns the account details for the logged-in user.
        :rtype: json
        :raises: MatchbookAPI.bin.exceptions.ApiError

        """
        method = 'account'
        resource = resources.AccountDetails
        if balance_only:
            method = 'account/balance'
            resource = resources.AccountBalance
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", self.client.urn_edge, method, session=session)
        date_time_received = datetime.datetime.utcnow()
        return self.process_response(response.json(), resource, date_time_sent, date_time_received)

    def wallet_transfer(self, amount, session=None):
        #TODO: Populate Acccount Transfer Resource.
        """
        Transfer balance from one 
        
        :param amount: amount to be transferred, >0 for casino to sports transfer, <0 for the opposite.
        :type amount: float
        :param session: requests session to be used.
        :type session: requests.Session
        :return: details of the success/failure of the transfer.
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        date_time_received = datetime.datetime.utcnow()
        response = self.request("POST", self.client.urn_main, 'account/transfer', data=params, session=session)
        return self.process_response(response.json(), resources.AccountTransfer, date_time_sent, date_time_received)

    def get_casino_balance(self, session=None):
        """
        Get casino account balance for logged in user.

        :param balance_only: retrieve only account balance info subset or not.
        :type balance_only: bool
        :param session: requests session to be used.
        :type session: requests.Session
        :returns: Returns the casino balance for the logged-in user.
        :rtype: json
        :raises: MatchbookAPI.bin.exceptions.ApiError

        """
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", self.client.urn_main, 'account/balance', session=session)
        date_time_received = datetime.datetime.utcnow()
        return self.process_response(response.json(), resources.AccountBalance, date_time_sent, date_time_received)




import json

from matchbook.endpoints.baseendpoint import BaseEndpoint
from matchbook.exceptions import AuthError
from matchbook.utils import logger


class KeepAlive(BaseEndpoint):

    def __call__(self, session=None):
        session = session or self.client.session
        response = self.request('GET', self.client.urn_main, 'security/session', data=self.data, session=session)
        if response.status_code == 200:
            logger.info('Session Active, Session Token: %s' % self.client.session_token)
        elif response.status_code == 401:
            logger.info('Session Expired, Creating New Session')
            response = self.request("POST", self.client.urn_main, 'security/session', data=self.data, session=session)
            if response.status_code == 200:
                response_json = response.json()
                self.client.set_session_token(response_json.get('session-token'), response_json.get('user-id'))
                logger.info('Login Successful. Session Token: {}'.format(self.client.session_token))
            else:
                raise AuthError(response)
        else:
            raise AuthError(response)

    def request(self, request_method, urn, method, params={}, data={}, target=None, session=None):
        """
        :param request_method: type of request to be sent.
        :param urn: matchbook urn to append to url specified.
        :param method: Matchbook method to be used.
        :param params: Params to be used in request
        :param url: define different URL to use.
        :param data: data to be sent in request body.
        :param target: target key to get from data, if None returns full response.
        :param session: Requests session to be used, reduces latency.
        """
        session = session or self.client.session
        data['session-token'] = self.client.session_token
        data['user-id'] = self.client.user_id
        request_url = '%s%s%s' % (self.client.url, urn, method)
        response = session.request(
            request_method, request_url, params=params, data=json.dumps(data), headers=self.client.headers
        )
        return response

    @property
    def data(self):
        return {'username': self.client.username, 'password': self.client.password}


import pandas as pd
from pandas.io.json import json_normalize
from matchbook.utils import clean_time
import numpy as np
import re


def clean_bet_report(df):
    """
    Clean raw data returned from MatchbookAPI.MatchbookAPI.get_bets_report

    :param df: MatchbookAPI.MatchbookAPI.get_bets_report raw data.
    :type df: Pandas.Dataframe
    :returns: cleaned bet report.
    :rtype: Pandas.Dataframe

    """
    if 'bets' in df.columns:
        df = df.pipe(parser_function, sculpt_col, 'bets')
        df.rename(columns={'id':'offer-id'}, inplace=True)
    for _ in ['submitted-at', 'placed-at', 'settled-at']:
        if _ in df.columns:
            df[_] = clean_time(df[_])
    return df


def clean_commission_report(df):
    """
    Clean raw data returned from MatchbookAPI.MatchbookAPI.get_commission_report

    :param df: MatchbookAPI.MatchbookAPI.get_commission_report raw data.
    :type df: Pandas.Dataframe
    :returns: cleaned commission report.
    :rtype: Pandas.Dataframe

    """
    if 'sub-totals' in df.columns:
        df = df.pipe(parser_function, sculpt_col, 'sub-totals')
    if 'settled-at' in df.columns:
        df['settled-at'] = clean_time(df['settled-at'])
    if 'placed-at' in df.columns:
        df['placed-at'] = clean_time(df['placed-at'])
    return df


def clean_transaction_report(df):
    """
    Clean raw data returned from MatchbookAPI.MatchbookAPI.get_transactions_report

    :param df: MatchbookAPI.MatchbookAPI.get_transactions_report raw data.
    :type df: Pandas.Dataframe
    :returns: cleaned transactions report.
    :rtype: Pandas.Dataframe

    """
    if 'transactions' in df.columns:
        df = df.pipe(parser_function, sculpt_col, 'transactions')
    return df


def clean_orders(df):
    """
    Clean raw data returned from MatchbookAPI.MatchbookAPI.get_orders

    :param df: MatchbookAPI.MatchbookAPI.get_orders raw data.
    :type df: Pandas.Dataframe
    :returns: cleaned orders report.
    :rtype: Pandas.Dataframe

    """
    if 'offers' in df.columns:
        df = df.pipe(parser_function, sculpt_col, 'offers')
        df.rename(columns={'id': 'offer-id'}, inplace=True)
    if 'matched-bets' in df.columns:
        df = df.pipe(parser_function, sculpt_col, 'matched-bets')
        df.rename(columns={'id': 'bet-id'}, inplace=True)
    if 'created-at' in df.columns:
        df['created-at'] = clean_time(df['created-at'])
    return df


def clean_settlement_report(df):
    """
    Clean raw data returned from MatchbookAPI.MatchbookAPI.get_settlement_report

    :param df: MatchbookAPI.MatchbookAPI.get_settlement_report raw data.
    :type df: Pandas.Dataframe
    :returns: cleaned settlement report.
    :rtype: Pandas.Dataframe

    """
    if 'markets' in df.columns:
        df = df.pipe(parser_function, sculpt_col, 'markets')
    if 'runners' in df.columns:
        df = df.pipe(parser_function, sculpt_col, 'runners')
    if 'settled-at' in df.columns:
        df['settled-at'] = clean_time(df['settled-at'])
    return df


def parse_meta_tags(df):
    """
    Clean raw data returned from MatchbookAPI.MatchbookAPI.get_navigation

    :param df: MatchbookAPI.MatchbookAPI.get_navigation raw data.
    :type df: Pandas.Dataframe
    :returns: cleaned navigation data.
    :rtype: Pandas.Dataframe

    """
    if 'meta-tags' in df.columns:
        df = df.pipe(parser_function, sculpt_col, 'meta-tags')
        df.rename(columns={'id': 'tree-id1', 'name': 'tree-name1', 'url-name': 'tree-url1'}, inplace=True)
    if 'meta-tags' in df.columns:
        df = df.pipe(parser_function, sculpt_col, 'meta-tags')
        df.rename(columns={'id': 'type1-id', 'type': 'type1', 'name': 'type1-name', 'url-name': 'type1-url-name'},
                  inplace=True)
    if 'tree-id' in df.columns:
        df.drop('tree-id', axis=1, inplace=True)
    if 'meta-tags' in df.columns:
        df = df.pipe(parser_function, sculpt_col, 'meta-tags')
        df.rename(columns={'id': 'type2-id', 'type': 'type2', 'name': 'type2-name', 'url-name': 'type2-url-name'},
                  inplace=True)
    if 'meta-tags' in df.columns:
        df = df.pipe(parser_function, sculpt_col, 'meta-tags')
        df.rename(columns={'id': 'type3-id', 'type': 'type3', 'name': 'type3-name', 'url-name': 'type3-url-name'},
                  inplace=True)
    if 'meta-tags' in df.columns:
        df = df.pipe(parser_function, sculpt_col, 'meta-tags')
    return df


def split_pricedata(df):
    """
    Splits prices dictionary into individual columns by level.

    :param df: dataframe containing the pricing information to split.
    :type df: Dataframe
    :returns: pricing information.
    :rtype: Pandas.Series
    """
    p_d = {}
    back_count = 0
    lay_count = 0
    for price in df.prices:
        if price['side'] == 'back':
            back_count += 1
            p_d['Back' + str(back_count)] = price['odds']
            p_d['BackSize' + str(back_count)] = price['available-amount']
            #p_d['Back' + str(price['odds'])] = price['available-amount']
        elif price['side'] == 'lay':
            lay_count += 1
            p_d['Lay' + str(lay_count)] = price['odds']
            p_d['LaySize' + str(lay_count)] = price['available-amount']
            #p_d['Lay' + str(price['odds'])] = price['available-amount']
        else:
            continue
    return pd.Series(p_d)


def clean_event_col_types(df):
    """
    Clean specific columns returned from events requests.

    :param df: data returned from events requests.
    :type df: Pandas.Dataframe
    :returns: data with clean start, handicap and asian-handicap columns.
    :rtype: Pandas.Dataframe

    """
    all_cols = df.columns
    if 'start' in all_cols:
        df['start'] = clean_time(df['start'])
    if 'handicap' in all_cols:
        df['handicap'] = df.handicap.apply(lambda x: float(x) if x != '' else np.nan)
    if 'asian-handicap' in all_cols:
        df['asian-handicap'] = df['asian-handicap'].apply(lambda x:
                                                          np.mean([float(y) for y in re.findall('\d\.\d', x)]) *
                                                          (-1 if '-' in x else 1))
    return df


def parse_event_data(df):
    """
    Clean raw data returned from MatchbookAPI.MatchbookAPI.get_events or MatchbookAPI.MatchbookAPI.get_event_allmarkets.

    :param df: MatchbookAPI.MatchbookAPI.get_events or MatchbookAPI.MatchbookAPI.get_event_allmarkets raw data.
    :type df: Pandas.Dataframe
    :returns: cleaned events data breakdown.
    :rtype: Pandas.Dataframe

    """
    if 'events' in df.columns:
        df = df.pipe(parser_function, sculpt_col, 'events')
    df.rename(columns={'id': 'event-id', 'name': 'event-name'}, inplace=True)
    if 'category-id' in df.columns:
        df['category-id'] = df['category-id'].apply(lambda x: x[0])
    if 'meta-tags' in df.columns:
        df.drop('meta-tags', axis=1, inplace=True)
    if 'markets' in df.columns:
        df = df.pipe(parser_function, sculpt_col, 'markets')
        df.rename(columns={'id': 'market-id', 'name': 'market-name'}, inplace=True)
    if 'runners' in df.columns:
        df = df.pipe(parser_function, sculpt_col, 'runners')
        df.rename(columns={'id': 'runner-id', 'name': 'runner-name'}, inplace=True)
    if 'prices' in df.columns:
        df = pd.concat([df, df.apply(split_pricedata, axis=1)], axis=1)
        df.drop('prices', axis=1, inplace=True)
    return df


def parse_single_market(df):
    """
    Clean raw data returned from MatchbookAPI.MatchbookAPI.get_event_singlemarket or MatchbookAPI.MatchbookAPI.get_event_marketrunners.

    :param df: MatchbookAPI.MatchbookAPI.get_event_singlemarket or MatchbookAPI.MatchbookAPI.get_event_marketrunners raw data.
    :type df: Pandas.Dataframe
    :returns: cleaned events data breakdown.
    :rtype: Pandas.Dataframe

    """
    df.rename(columns={'id': 'market-id', 'name': 'market-name'}, inplace=True)
    if 'runners' in df.columns:
        df = df.pipe(parser_function, sculpt_col, 'runners')
        df.rename(columns={'id': 'runner-id', 'name': 'runner-name'}, inplace=True)
    if 'prices' in df.columns:
        df = pd.concat([df, df.apply(split_pricedata, axis=1)], axis=1)
        df.drop('prices', axis=1, inplace=True)
    return df


def parse_single_runner(df):
    """
    Clean raw data returned from MatchbookAPI.MatchbookAPI.get_event_singlerunner.

    :param df: MatchbookAPI.MatchbookAPI.get_event_singlerunner raw data.
    :type df: Pandas.Dataframe
    :returns: cleaned events data breakdown.
    :rtype: Pandas.Dataframe

    """
    df.rename(columns={'id': 'runner-id', 'name': 'runner-name'}, inplace=True)
    if 'prices' in df.columns:
        df = pd.concat([df, df.apply(split_pricedata, axis=1)], axis=1)
        df.drop('prices', axis=1, inplace=True)
    return df


def parser_function(df, parsing_function, col_name):
    """
    Parse a specified dataframe column when provided with parsing breakdowns to use.

    :param df: dataframe which contains the column to be parsed
    :type df: Dataframe
    :param parsing_function: function to parse the specified col_name using.
    :type parsing_function: func
    :param col_name: column name in df that is to be parsed.
    :returns: flattened dataframe
    :rtype: Dataframe

    """
    df[col_name] = df[col_name].map(lambda x: pd.DataFrame() if len(x) == 0 else json_normalize(x))
    df = df.apply(parsing_function, args=(col_name,), axis=1)
    df = pd.concat([x for x in df[col_name].tolist()], ignore_index=True)
    df = df.fillna('')
    return df


def sculpt_col(df, col_name):
    """
    Flattens Dataframe where column is of type dataframe, applied cell by cell.

    :param df:
    :type df: dataframe row
    :param col_name: name of the column which is to be flattened.
    :type col_name: str
    :returns: flattened dataframe
    :rtype: Dataframe

    """
    try:
        if len(df[col_name]) > 0:
            for key in df.keys().tolist():
                if key != col_name:
                    df[col_name][key] = df[key]
        else:
            keys = df.keys().tolist()
            keys.pop(keys.index(col_name))
            df[col_name] = pd.DataFrame(columns=[key for key in keys], data=[[df[key] for key in keys]])
    except:
        raise ValueError(df[col_name])
    return df



from matchbook.resources.baseresource import BaseResource


class AccountDetails(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'id': 'id',
            'balance': 'balance',
            'bet-slip-pinned': 'bet-slip-pinned',
            'commission-reserve': 'commission-reserve',
            'commission-type': 'commission-type',
            'currency': 'currency',
            'email': 'email',
            'exchange-type': 'exchange-type',
            'exposure': 'exposure',
            'family-names': 'family-names',
            'free-funds': 'free-funds',
            'given-names': 'given-names',
            'language': 'language',
            'odds-type': 'odds-type',
            'roles': 'roles',
            'show-bet-confirmation': 'show-bet-confirmation',
            'show-odds-rounding-message': 'show-odds-rounding-message',
            'show-position': 'show-position',
            'status': 'status',
            'user-id': 'user-id',
            'username': 'username'
        }


class AccountBalance(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'id': 'id',
            'balance': 'balance',
            'commission-reserve': 'commission-reserve',
            'exposure': 'exposure',
            'free-funds': 'free-funds',
        }


class AccountTransfer(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {

        }




from setuptools import setup, find_packages
from matchbook import __version__

setup(
    name="matchbook",
    version=__version__,
    author="Rory Cole",
    author_email="rory.cole1990@gmail.com",
    description="Matchbook API Python wrapper",
    url="https://github.com/rozzac90/matchbook",
    packages=find_packages(),
    install_requires=[line.strip() for line in open("requirements.txt")],
    long_description=open('README.rst').read(),
    tests_require=['pytest'],
)



from matchbook.resources.baseresource import BaseResource


class MatchedBets(BaseResource):
    class Meta(BaseResource.Meta):
        identifier = 'matched-bets'
        attributes = {
            'commission': 'commission',
            'created-at': 'created-at',
            'currency': 'currency',
            'decimal-odds': 'decimal-odds',
            'id': 'id',
            'odds': 'odds',
            'odds-type': 'odds-type',
            'potential-profit': 'potential-profit',
            'stake': 'stake',
        }
        datetime_attributes = (
            'created-at'
        )


class Order(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'created-at': 'created-at',
            'currency': 'currency',
            'decimal-odds': 'decimal-odds',
            'event-id': 'event-id',
            'event-name': 'event-name',
            'exchange-type': 'exchange-type',
            'id': 'id',
            'market-id': 'market-id',
            'market-name': 'market-name',
            'odds': 'odds',
            'odds-type': 'odds-type',
            'potential-profit': 'potential-profit',
            'remaining': 'remaining',
            'remaining-potential-profit': 'remaining-potential-profit',
            'runner-id': 'runner-id',
            'runner-name': 'runner-name',
            'side': 'side',
            'stake': 'stake',
            'status': 'status',
        }
        sub_resources = {
            'matched-bets': MatchedBets,
        }
        datetime_attributes = (
            'created-at'
        )


class Position(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {

        }